/******************************************************************************
*
* project name:    TI-Chess
* file name:       tichess.c
* initial date:    20/02/2000
* authors:         thomas.nussbaumer@gmx.net (coding)
*                  marcos.lopez@gmx.net      (design/graphics/beta testing)
* description:     the entry point and the special clearscreen routine
*
* $Id: tichess.c,v 1.7 2001/02/17 15:56:10 Thomas Nussbaumer Exp $
*
******************************************************************************/

//=============================================================================
// NOTE: in previous TIGCC version (<0.90) OPTIMIZE_ROM_CALLS doesn't work
//       correctly for own interrupt handlers
//
//       this bug seems to be fixed with TIGCC 0.90
//=============================================================================
#define OPTIMIZE_ROM_CALLS // makes ROM calls smaller and faster

#include <tigcclib.h>

//-----------------------------------------------------------------------------
// check for -DVERSION_TI89 commandline flag (for TI-89 version)
// *** or ***
// check for -DVERSION_TI92P commandline flag (for TI-92p version)
//
// ONLY ONE IS ALLOWED PER COMPILE RUN
//-----------------------------------------------------------------------------
#if defined(VERSION_TI89) && defined(VERSION_TI92P)
#error cannot produce both versions at once
#elif defined(VERSION_TI89)
short _ti89;     // produce TI-89 code (89z file)
#elif defined(VERSION_TI92P)
short _ti92plus; // produce TI-92p code (9xz file)
#else
#error Please specify either -DVERSION_TI89 or -DVERSION_TI92P on commandline
#endif

//-----------------------------------------------------------------------------
// useful macro for calculator type definitions
//-----------------------------------------------------------------------------
#if defined(VERSION_TI89)
#define C89_92(x,y) (x)
#else
#define C89_92(x,y) (y)
#endif


#include "version.h"
#include "tichess.h"
#include "logout.h"      // contains macros for log writing to link port
#include "interrupt.h"   // own key handler stuff
#include "input.h"       // key definitions
#include "generic.h"     // generic graphics function definitions


//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
// due to a very strange behavior of TIGCC we cannot use separate object until
// yet (the globals are not initialized correctly - NOT even the constants !!)
//
// So we've to go that really silly "include all into one" way ...
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

unsigned short *light_plane = 0;
unsigned short *dark_plane  = 0;
SCR_RECT fullscreen         = {{0, 0, 239, 127}}; // used for fullscreen clipping
short    grayadjust_val     = 0;

#include "waitms.c"     // WaitForMillis() utility function
#include "interrupt.c"  // own key handler stuff
#include "generic.c"    // generic drawing functions
#include "board.c"      // pieceset data and output routine for single square
#include "infoboards.c" // handling routines for infoboards
#include "clocks.c"     // clocks data and routines
#include "input.c"      // user input handling
#include "logo.c"       // splashscreen data and output routine

#if defined(VERSION_TI89)
#include "bg_ti89.c"    // background data and output routine
#else
#include "bg_ti92.c"    // background data and output routine
#endif

#include "loadsave.c"   // loading/saving stuff
#include "gui.c"        // gui stuff
#include "engine.c"     // engine stuff
#include "hash.c"       // hashtable stuff
#include "menu.c"       // menu stuff

#if ENABLE_LOGGING
#include "logout.c"   // includes logging routines if necessary
#endif


//--------------------------------------------------
// declarations of the necessary "external" routines
//--------------------------------------------------
extern char default_hashmode;




/*===========================================================================*/
/* special effect clear screen                                               */
/*===========================================================================*/
void FXClearScreen(void) {
    short height  = LCD_HEIGHT;
    short height2 = height/2;
    short width   = LCD_WIDTH;
    short width2  = width/2;
    short i,j;

    //---------------------
    // clear in y direction
    //---------------------
    for (i=0;i<=height2;i++) {
        DrawLine(0,i,width,i,A_REPLACE);
        DrawLine(0,height-i,width,height-i,A_REPLACE);
        DrawLine(0,i-1,width,i-1,A_REVERSE);
        DrawLine(0,height-i+1,width,height-i+1,A_REVERSE);
        for (j=0;j<=10000;j++);
    }

    //-----------------------------
    // clear in x direction
    //-----------------------------
    for (i=0;i<=width2;i++) {
        DrawPix(i,height2,A_REVERSE);
        DrawPix(width-i,height2,A_REVERSE);
        for (j=0;j<=3000;j++);
    }
}



/*===========================================================================*/
/* due to a real strange behaviour the contrast gets lost when we set our    */
/* keyboard handler !!!                                                      */
/* I haven't managed to restore it correctly in the interrupt installer so I */
/* will do it just in the main routine ...                                   */
/*                                                                           */
/* The following routine fetches the address where the contrast will be      */
/* stored, so we can get and restore it later again                          */
/*===========================================================================*/
asm("ActiveContrastAddr: movem.l %d0-%d1,-(%a7)
         move.w          #4,%d0
         trap            #9
         movem.l (%a7)+,%d0-%d1
         rts");



/*===========================================================================*/
/* MAIN - where all the fun starts ...                                       */
/*===========================================================================*/
void _main(void) {
    char*          s1 = "ERROR: Memory";
    char*          s2 = "Insufficient Memory.";
    char           s3[30];
    unsigned short size;
    unsigned char  initial_contrast;
    LCD_BUFFER     backbuffer;

#if defined(VERSION_TI92P)
    if (TI89) {
        ST_showHelp("Cannot run TI-92p version");
        return;
    }
#else
    if (!TI89) {
        ST_showHelp("Cannot run TI-89 version");
        return;
    }
#endif
    LCD_save(backbuffer);

    initial_contrast = *ActiveContrastAddr(); // get the initial contrast

    LoadDefaults();   // load defaults from ticcfg

    default_hashmode = defaults.hashtables;

    FXClearScreen();

#if SHOW_MEM_USAGE
    if (default_hashmode == OPTION_ON) size = sizeof(hentry_t)*HASHSIZE*2;
    else                               size = 0;

    size += sizeof(char)*BOARD_SIZE + \
            sizeof(short)*MAX_DEPTH+sizeof(move_t)*(STACK_SIZE+1) + \
            sizeof(short)*MAX_DEPTH+sizeof(killer_t)*MAX_DEPTH + \
            sizeof(bicolor_t)*BOARD_SIZE + \
            sizeof(bicolor_t)*(H_COLUMN+2)*2 + \
            sizeof(short)*MAX_DEPTH*4 + \
            sizeof(fromto_t*)*MAX_DEPTH + \
            sizeof(fromto_t)*MAX_DEPTH*MAX_DEPTH + \
            sizeof(hash_t*)*12 + \
            sizeof(hash_t)*64*12 + \
            sizeof(store_t)*MAX_MOVES_STORED;
#endif
    FontSetSys(F_6x8);

    if (!InitMemory()) {
        FreeMemory();

        size = sizeof(char)*BOARD_SIZE + \
           sizeof(short)*MAX_DEPTH+sizeof(move_t)*(STACK_SIZE+1) + \
           sizeof(short)*MAX_DEPTH+sizeof(killer_t)*MAX_DEPTH + \
           sizeof(bicolor_t)*BOARD_SIZE + \
           sizeof(bicolor_t)*(H_COLUMN+2)*2 + \
           sizeof(short)*MAX_DEPTH*4 + \
           sizeof(fromto_t*)*MAX_DEPTH + \
           sizeof(fromto_t)*MAX_DEPTH*MAX_DEPTH + \
           sizeof(hash_t*)*12 + \
           sizeof(hash_t)*64*12 + \
           sizeof(store_t)*MAX_MOVES_STORED;

        sprintf(s3,"(Req'd %u Bytes)",size);

        FontSetSys(F_8x10);
        DrawStr((ScrRect->xy.x1-DrawStrWidth(s1,F_8x10))/2,ScrRect->xy.y1/3,s1,A_REPLACE);
        FontSetSys(F_6x8);
        DrawStr((ScrRect->xy.x1-DrawStrWidth(s2,F_6x8))/2,ScrRect->xy.y1/2,s2,A_REPLACE);
        DrawStr((ScrRect->xy.x1-DrawStrWidth(s3,F_6x8))/2,ScrRect->xy.y1*2/3,s3,A_REPLACE);
        ngetchx();
        LCD_restore(backbuffer);
        return;
    }

#if SHOW_MEM_USAGE
    sprintf(s3,"[using %u Bytes]",size);
    FontSetSys(F_6x8);
    DrawStr(0,0,s3,A_REPLACE);
    DrawStr(0,10,"press a key",A_REPLACE);
    ngetchx();
#endif

    //----------------------------------------------------------
    // install own key handler (MUST BE DONE BEFORE GRAYMODE ON)
    //----------------------------------------------------------
    InstallInterruptHandlers();

    //---------------------------
    // turn on grayscale graphics
    //---------------------------
    if (!GrayMode(GRAY_ON)) {
        SaveDefaults();
        RestoreInterruptHandlers();
        LCD_restore(backbuffer);
        return;
    }

    //---------------------------------
    // !!!!! adjust grayscales !!!!!
    //---------------------------------
    grayadjust_val=0;
    GrayAdjust(0);

    //---------------------------------
    // !!!!! restore the contrast !!!!!
    //---------------------------------
    *ActiveContrastAddr() = initial_contrast;

    //-----------------------------------------
    // setup bitplane globals and show the logo
    //-----------------------------------------
    light_plane = GetPlane(LIGHT_PLANE);
    dark_plane  = GetPlane(DARK_PLANE);

    OSFreeTimer(USER_TIMER); // make sure timer is not in use !!

    Logo();

    //-----------------------------------------------
    // start timer for clocks and enter the main loop
    //-----------------------------------------------
    OSRegisterTimer(USER_TIMER,20);

    MainGUI();

    //-----------------------------------------------
    // stop timer again, turn grayscales off, restore
    // old interrupt handlers again and free memory
    //-----------------------------------------------
    OSFreeTimer(USER_TIMER);
    GrayMode(GRAY_OFF);
    RestoreInterruptHandlers();
    FreeMemory();

    //------------------------------------------
    // give the system keyboard handler time to
    // recover and then read out all keypresses
    //------------------------------------------
    WaitForMillis(200);
    GKeyFlush();              // remove any pending keys
    LCD_restore(backbuffer);
}



//#############################################################################
//###################### NO MORE FAKES BEYOND THIS LINE #######################
//#############################################################################
//
//=============================================================================
// Revision History
//=============================================================================
//
// $Log: tichess.c,v $
// Revision 1.7  2001/02/17 15:56:10  Thomas Nussbaumer
// cannot start ti89 version on ti92p and ti92p version on ti89 to
// prevent complete "hang"
//
// Revision 1.6  2001/02/17 15:00:11  Thomas Nussbaumer
// changes due to new TIGCC version
//
// Revision 1.5  2000/12/19 13:55:29  Thomas Nussbaumer
// warnings stated by compiling with option -Wall fixed
//
// Revision 1.4  2000/12/09 15:39:35  Thomas Nussbaumer
// not using OPTIMIZE_ROM_CALLS anymore (causes crashes)
//
// Revision 1.3  2000/12/02 15:22:00  Thomas Nussbaumer
// adaptions due to new TIGCC standard library 2.22 features
//
// Revision 1.2  2000/08/12 15:31:13  Thomas Nussbaumer
// substitution keywords added
//
//
