/******************************************************************************
*
* project name:    TI-Chess
* file name:       tichess.h
* initial date:    20/02/2000
* authors:         thomas.nussbaumer@gmx.net (coding)
*                  marcos.lopez@gmx.net      (design/graphics/beta testing)
* description:     project header file
*
* $Id: tichess.h,v 1.3 2001/02/17 15:00:11 Thomas Nussbaumer Exp $
*
******************************************************************************/

#ifndef TICHESS_H
#define TICHESS_H


/*---------------------------------------------------------------------------*/
/*  some defines for debugging                                               */
/*---------------------------------------------------------------------------*/
#define ENABLE_LOGGING    0   // used to enable log outputs to link port
#define SHOW_MEM_USAGE    0   // show memory usage at program start
#define ENABLE_TESTKEY    0   // if this is set to 1 [=] is queried, too



/*---------------------------------------------------------------------------*/
/*  calculator strength setting (don't modify this !!)                       */
/*---------------------------------------------------------------------------*/
#define MAX_USERLEVEL 5       // external level 4
#define STD_USERLEVEL 1       // external level 0
#define MIN_USERLEVEL 1       // external level 0



/*---------------------------------------------------------------------------*/
/*  generic constants                                                        */
/*---------------------------------------------------------------------------*/
#define BOARD_SIZE   120     /* dimension of extended chess board            */
#define MAX_DEPTH     20     /* maximum search depth                         */
#define NR_DIRS       16     /* number of directions of all figures          */
#define NR_FIGS        7     /* number of different figures (including NONE) */
#define STACK_SIZE  1000     /* size of move stack                           */



/*---------------------------------------------------------------------------*/
/*  figure constants                                                         */
/*---------------------------------------------------------------------------*/
#define B_KING     -6          /* black figures      */
#define B_QUEEN    -5
#define B_KNIGHT   -4
#define B_BISHOP   -3
#define B_ROOK     -2
#define B_PAWN     -1
#define EMPTY       0          /* empty field        */
#define W_PAWN      1          /* white figures      */
#define W_ROOK      2
#define W_BISHOP    3
#define W_KNIGHT    4
#define W_QUEEN     5
#define W_KING      6
#define OUTSIDE   100          /* outside real board */



/*---------------------------------------------------------------------------*/
/*  figure values                                                            */
/*---------------------------------------------------------------------------*/
#define VALUE_PAWN        100
#define VALUE_ROOK        500
#define VALUE_BISHOP      350
#define VALUE_KNIGHT      325
#define VALUE_QUEEN       900
#define VALUE_KING          0
#define VALUE_MATE      32000
#define MAX_POS         32000



/*---------------------------------------------------------------------------*/
/*  bonus values for main variant and killer moves                           */
/*---------------------------------------------------------------------------*/
#define HVARBONUS         500
#define KILLER1BONUS      250
#define KILLER2BONUS      150



/*---------------------------------------------------------------------------*/
/*  maximum value of materials                                               */
/*---------------------------------------------------------------------------*/
#define VALUE_MAX     ((4*(VALUE_ROOK+VALUE_BISHOP+VALUE_KNIGHT))+(2*VALUE_QUEEN))
#define VALUE_ENDGAME (4*VALUE_ROOK+(2*VALUE_BISHOP))



/*---------------------------------------------------------------------------*/
/* field names for often used fields (increasing readability)                */
/*---------------------------------------------------------------------------*/
#define  A1    21
#define  B1    22
#define  C1    23
#define  D1    24
#define  E1    25
#define  F1    26
#define  G1    27
#define  H1    28
#define  C2    33
#define  H2    38
#define  A3    41
#define  C3    43
#define  D3    44
#define  E3    45
#define  A6    71
#define  C6    73
#define  D6    74
#define  E6    75
#define  A7    81
#define  C7    83
#define  H7    88
#define  A8    91
#define  B8    92
#define  C8    93
#define  D8    94
#define  E8    95
#define  F8    96
#define  G8    97
#define  H8    98



/*---------------------------------------------------------------------------*/
/* line and column names                                                     */
/*---------------------------------------------------------------------------*/
#define A_COLUMN   1
#define B_COLUMN   2
#define C_COLUMN   3
#define D_COLUMN   4
#define E_COLUMN   5
#define F_COLUMN   6
#define G_COLUMN   7
#define H_COLUMN   8
#define ROW_1      2
#define ROW_2      3
#define ROW_3      4
#define ROW_4      5
#define ROW_5      6
#define ROW_6      7
#define ROW_7      8
#define ROW_8      9



/*---------------------------------------------------------------------------*/
/* rochade numbers                                                           */
/*---------------------------------------------------------------------------*/
#define ROCHADE_NONE  0
#define ROCHADE_SHORT 1
#define ROCHADE_LONG  2



/*---------------------------------------------------------------------------*/
/* color numbers                                                             */
/*---------------------------------------------------------------------------*/
#define WHITE    1
#define BLACK   -1



/*---------------------------------------------------------------------------*/
/* logic values                                                              */
/*---------------------------------------------------------------------------*/
#define TRUE           1
#define FALSE          0
#define LEGAL          1
#define ILLEGAL        0
#define OK             1



/*---------------------------------------------------------------------------*/
/* data type for hashcodes (unsigned 64bit)                                  */
/*---------------------------------------------------------------------------*/
typedef unsigned long long hash_t;



/*---------------------------------------------------------------------------*/
/* data type for move stack                                                  */
/*---------------------------------------------------------------------------*/
typedef struct {
   char  from;           /* from field               */
   char  to;             /* to field                 */
   char  killed_fig;     /* killed figure            */
   char  promoted_fig;   /* promoted figure          */
   char  rochade_nr;     /* kind of rochade          */
   char  ep_field;       /* enpassant field          */
   short value;          /* value for move sorting   */
   unsigned char count;  /* count for 50 ply rule    */
                         /* No pawn moved or figure  */
                         /* captured                 */
                         /* NOTE: 0x80 will be used  */
                         /* for side marking         */
   unsigned char unused; /* unused - but for padding */
} move_t;



/*---------------------------------------------------------------------------*/
/* data type for stored moves (extended move_t)                              */
/*---------------------------------------------------------------------------*/
typedef struct {
   char  from;           /* from field              */
   char  to;             /* to field                */
   char  killed_fig;     /* killed figure           */
   char  promoted_fig;   /* promoted figure         */
   char  rochade_nr;     /* kind of rochade         */
   char  ep_field;       /* enpassant field         */
   short value;          /* value for move sorting  */
   unsigned char count;  /* count for 50 ply rule   */
                         /* No pawn moved or figure */
                         /* captured                */
                         /* NOTE: 0x80 will be used */
                         /* for side marking        */
   char figure;          /* for display moves       */
                         /* OVERLAPS WITH UNUSED    */
   unsigned short flags; /* system flags            */
   short  sum;           /* stores material sum     */
   short  bal;           /* stores material balance */
   short  epfield;       /* stores epfield          */
   hash_t key;           /* for repetition checking */
} store_t;



/*---------------------------------------------------------------------------*/
/* index of figures in offset list and long/short marker (used by move gen.) */
/*---------------------------------------------------------------------------*/
typedef struct {
   char start;
   char end;
   char longmove;
} figoffset_t;



/*---------------------------------------------------------------------------*/
/* information about pawn/figure constellation                               */
/*---------------------------------------------------------------------------*/
typedef struct {
   char white;
   char black;
} bicolor_t;



/*---------------------------------------------------------------------------*/
/* information about from/to field                                           */
/*---------------------------------------------------------------------------*/
typedef struct {
   char from;
   char to;
} fromto_t;



/*---------------------------------------------------------------------------*/
/* data structure for killer moves                                           */
/*---------------------------------------------------------------------------*/
typedef struct {
   fromto_t killer1;
   fromto_t killer2;
} killer_t;



/*---------------------------------------------------------------------------*/
/* data structure of an hashtable entry                                      */
/* due to some tests I'm using now absolutely simple hashing                 */
/*---------------------------------------------------------------------------*/
typedef struct {
    hash_t        key;    // hashcode of position (64bit)
    short         val;    // calculated value (16bit)
} hentry_t;

#define MAX_MOVES_STORED    330   // size of move store
                                  // this have to be a minimum of
                                  // 101 + MAX_MOVES_REMOVED to make sure the
                                  // that the 50 ply (100 moves) rule may
                                  // work correctly
#define MAX_MOVES_REMOVED    30   // size of removed moves if too much moves

#define HASHSIZE 1024  // size of a hashtable (2 are used)
                       // MUST BE A POWER OF 2 !!!

#define B1_MOVE_MASK      0x0001
#define C1_MOVE_MASK      0x0002
#define F1_MOVE_MASK      0x0004
#define G1_MOVE_MASK      0x0008
#define W_ROCHADE_MASK    0x0010

#define B8_MOVE_MASK      0x0020
#define C8_MOVE_MASK      0x0040
#define F8_MOVE_MASK      0x0080
#define G8_MOVE_MASK      0x0100
#define B_ROCHADE_MASK    0x0200

#define E1_MOVE_MASK      0x0400
#define E8_MOVE_MASK      0x0800

#define A1_MOVE_MASK      0x1000
#define H1_MOVE_MASK      0x2000
#define A8_MOVE_MASK      0x4000
#define H8_MOVE_MASK      0x8000

#define INIT_SYSTEM_FLAGS 0x0000

char developed_look_up[32] = {
  0,1,1,2,1,2,2,3,1,2,2,3,2,3,3,4,
  1,2,2,3,2,3,3,4,2,3,3,4,3,4,4,5
};

#define WHITE_DEVELOPED(flags) developed_look_up[(flags) & 0x001f]
#define BLACK_DEVELOPED(flags) developed_look_up[((flags) >> 5) & 0x001f]



/*---------------------------------------------------------------------------*/
/* data type for default settings and displayed menus                        */
/*---------------------------------------------------------------------------*/
typedef struct {
    unsigned char strength;   // default strength
    unsigned char autoload;   // autoload feature
    unsigned char autosave;   // autosave feature
    unsigned char hashtables; // hashtable usage
    unsigned char firsttime;  // PREVIOUSLY saved?
    unsigned char requests;
    unsigned char savemoves;
    unsigned char pieceset;
    unsigned char unused;     // just to make it uncompatible to config of
                              // TI-Chess 2.01
} defaults_t;


#define OPTION_ON  0
#define OPTION_OFF 1



/*---------------------------------------------------------------------------*/
/* defines for main menu entries                                             */
/*---------------------------------------------------------------------------*/
#define MENU_NONE        -1  // not a real index !!
#define MENU_NEW          0
#define MENU_LOADSAVE     1
#define MENU_PROBLEM      2
#define MENU_SETTINGS     3
#define MENU_HELP         4
#define MENU_ABOUT        5
#define MENU_EXIT         6
#define MENU_CMD_LOAD    98  // not a real index !!
#define MENU_CMD_SAVE    99  // not a real index !!
#define MENU_CMD_EXPORT 100  // not a real index !!
#define MENU_SHOWMOVES  101  // not a real index !!



/*---------------------------------------------------------------------------*/
/* defines for chess problems                                                */
/*---------------------------------------------------------------------------*/
#define SIZE_TEXT      22
#define SIZE_COPYRIGHT 32
#define SIZE_PROBLEM   (3*SIZE_TEXT+SIZE_COPYRIGHT+32+4)


//-----------------------------------------------------------------------------
// bg_ti89.c / bg_ti92.c
//-----------------------------------------------------------------------------
void OutputBackground();

//-----------------------------------------------------------------------------
// board.c
//-----------------------------------------------------------------------------
void Sprite16x12Gray(short x,short y,unsigned short* sprite1,unsigned short* sprite2,short mode);
void DrawFigure(short x, short y,char s);
void DrawSquare(short x, short y,char s);
void DrawCoordinates(short inverted);

//-----------------------------------------------------------------------------
// clocks.c
//-----------------------------------------------------------------------------
void Sprite8x7Gray(short x,short y,unsigned char *sprite1,unsigned char *sprite2,short mode);
void DrawNumberOfClock(short nr,short clocknr);
void DrawPointsOfClock(short clocknr);
void UpdateAllNumbers(short nr);
void IncrementClock(void);
void RedrawClocks(void);
void SwitchClockPositions(void);
void ToggleClocks(short reset,short inverted);

//-----------------------------------------------------------------------------
// engine.c
//-----------------------------------------------------------------------------
short CheckForRepetition(short exact);
void  StoreMove(short i);
void  TakeBackMove(short silent);
void  UndoTakeBackMove(short silent);
short InitMemory(void);
void  FreeMemory(void);
void  InitEngine(short level);
char  Figure2Char(char figure);
char* Field2Notation(short fieldnumber,char* fieldnotation);
short Notation2Field(char* fieldnotation);
char* Move2Str(move_t *pmove,short fromboard);
void  InitTree(void);
short NextBestMove(void);
short AlphaBetaSearch(short alpha,short beta,short distance);
void  CopyMainVariant(short idx);
void  HandleMoveFlags(short field,unsigned short* flags);
void  HandleKilledFigure(char figure,short field,short increment);
void  HandleFromField(char figure,char promoted,short from,short to,short increment);
void  ExecMove(short idx);
void  UndoMove(short idx);
inline short BlackPawnEvaluation(short field,short row, short column,short developed);
inline short WhitePawnEvaluation(short field,short row, short column,short developed);
short Evaluate(short side);
void  PutMoveOnStack(short from,short to);
void  PutKillMoveOnStack(short from,short to);
void  PutEpOnStack(short from,short to,short epfield);
void  PutPromotionOnStack(short from,short to);
short AttacksField(short field, short side);
void  GenerateMoves(short allmoves);
void  InitPawnVars(void);
move_t* ComputerMove(void);
short UserMove(char* input,short updateboards);

//-----------------------------------------------------------------------------
// generic.c
//-----------------------------------------------------------------------------
void DrawColorLine(short x0,short y0,short x1, short y1, short color);
void DrawColorRect(short x0,short y0,short x1, short y1, short color,short fill);
void InvertGrayRect(short x0,short y0,short x1, short y1);
void DrawPopup(short x0,short y0,short x1, short y1);
void DrawString(short x,short y,char* s,short font,short attr);

//-----------------------------------------------------------------------------
// gui.c
//-----------------------------------------------------------------------------
void ResetDisplays(void);
void DrawMarkingRect(short idx);
short Cursor2Idx(short x,short y);
void DrawBoard(short hidecursor);
void InitGUI(void);
short RequestDialog(char* request);
void LeavePrg(char* msg);
void CorrectInfoBoard(void);
void HandleComputerMove(void);
void MajorInputLoop(void);
void MainGUI(void);

//-----------------------------------------------------------------------------
// hash.c
//-----------------------------------------------------------------------------
void ResetHashCounters(void);
inline short RetrieveFromHash(short *value);
inline void StoreInHash(short value);
unsigned long Random32(void);
void InitHashCode(void);
inline void XorHKey(char fig,short square);
void CalcHashKey(void);

//-----------------------------------------------------------------------------
// infoboard.c
//-----------------------------------------------------------------------------
void DrawInfoBoard(short nr);
void UpdateMoveDisplay(move_t* pmove,short fromboard);
void UpdateNodeDisplay(long nodes);
void SetActiveInfo(short color,short level,short redraw);
void OutputSpecial(char* s1,char* s2);
void SwitchInfoBoards(void);
void ClearActiveInfoBoardData(void);
void ClearInfoBoardData(void);
void InitInfo(short color,short level1,short level2,short inverted);

//-----------------------------------------------------------------------------
// input.c
//-----------------------------------------------------------------------------
short KeyPressed(void);
short GetUserInput(short inc_clock);

//-----------------------------------------------------------------------------
// loadsave.c
//-----------------------------------------------------------------------------
short isArchived(char* name);
short HandleLoad(short automatic,short load_problem);
short HandleSave(short automatic);
short LoadDefaults(void);
short SaveDefaults(void);
void LoadProblemDesc(void);
void HandleExport(void);

//-----------------------------------------------------------------------------
// logo.c
//-----------------------------------------------------------------------------
void Logo(void);

//-----------------------------------------------------------------------------
// menu.c
//-----------------------------------------------------------------------------
void InitCenterMenu(void);
void InvertOptionEntry(short index);
void OptionMenu(void);
void InvertLoadSaveEntry(short index);
short LoadSaveMenu(void);
void InvertMainEntry(short index);
void DrawMainMenu(void);
short MainMenu(short showmenu);
void HelpMenu(void);
void ShowMovesMenu(void);
void AboutMenu(void);
short ProblemMenu(void);
short PromotionDialog(void);

//-----------------------------------------------------------------------------
// tichess.c
//-----------------------------------------------------------------------------
void FXClearScreen(void);
unsigned char* ActiveContrastAddr(void);























#endif


//#############################################################################
//###################### NO MORE FAKES BEYOND THIS LINE #######################
//#############################################################################
//
//=============================================================================
// Revision History
//=============================================================================
//
// $Log: tichess.h,v $
// Revision 1.3  2001/02/17 15:00:11  Thomas Nussbaumer
// changes due to new TIGCC version
//
// Revision 1.2  2000/08/12 15:31:13  Thomas Nussbaumer
// substitution keywords added
//
//
